@@ -2,10 +2,10 @@ from __future__ import annotations
 
 import functools
 import itertools
-import string
 from collections import defaultdict
 
 from musictool import config
+from musictool.card import Card
 from musictool.chord import Chord
 from musictool.config import BLACK_BRIGHT
 from musictool.config import BLUE
@@ -16,7 +16,7 @@ from musictool.noteset import NoteSet
 from musictool.piano import Piano
 
 
-class Scale(NoteSet):
+class Scale(NoteSet, Card):
     intervals_to_name = {
         # diatonic
         frozenset({0, 2, 4, 5, 7, 9, 11}): 'major',
@@ -58,6 +58,7 @@ class Scale(NoteSet):
     }
     name_to_intervals = {v: k for k, v in intervals_to_name.items()}
     root: Note
+    name: str
 
     def __init__(
         self,
@@ -107,23 +108,30 @@ class Scale(NoteSet):
     def to_piano_image(self):
         return Piano(note_colors={note: config.scale_colors[scale] for note, scale in self.note_scales.items()})._repr_svg_()
 
-    def _repr_html_(self, html_classes: tuple[str, ...] = ('card',)) -> str:
-        if self.name is not None:
-            html_classes += self.name,
-        chords_hover = ''
+    def _repr_html_(
+        self,
+        html_classes: tuple[str, ...] = (),
+        title: str | None = None,
+        subtitle: str | None = None,
+        header_href: str | None = None,
+    ) -> str:
+        html_classes += self.name,
+
         if C_name := self.note_scales.get(Note('C'), ''):
             C_name = f' | C {C_name}'
-        return f"""
-        <div class='{' '.join(html_classes)}' {chords_hover}>
-        <a href='{self.root.name}'><span class='card_header'><h3>{self.root.name} {self.name}{C_name}</h3></span></a>
-        {self.to_piano_image()}
-        </div>
-        """
+
+        return self.repr_card(
+            html_classes=html_classes,
+            title=title or f'{self.root.name} {self.name}{C_name}',
+            subtitle=subtitle,
+            header_href=header_href or self.root.name,
+            piano_html=self.to_piano_image(),
+        )
 
 # flake8: noqa
 
 
-class ComparedScales:
+class ComparedScales(Card):
     """
     this is compared scale
     local terminology: left scale is compared to right
@@ -140,8 +148,6 @@ class ComparedScales:
         if right.kind == 'diatonic':
             self.shared_triads = frozenset(left.triads) & frozenset(right.triads)
 
-    # def __format__(self, format_spec): raise No
-
     def to_piano_image(self):
         return Piano(
             note_colors={note: config.scale_colors[scale] for note, scale in self.right.note_scales.items()},
@@ -158,35 +164,25 @@ class ComparedScales:
             } if self.right.kind == 'diatonic' else {},
         )._repr_svg_()
 
-    # @functools.cached_property
     def _repr_html_(
         self,
         html_classes: tuple[str, ...] = ('card',),
+        title: str | None = None,
         subtitle: str | None = None,
+        header_href: str | None = None,
     ) -> str:
+
         if C_name := self.right.note_scales.get(Note('C'), ''):
             C_name = f' | C {C_name}'
 
-        mapping = dict(
-            classes=' '.join(html_classes),
-            href=self.right.root.name,
-            title=f'<h3>{self.right.root.name} {self.right.name}{C_name}</h3>',
-            subtitle=subtitle if subtitle is not None else '',
-            piano=self.to_piano_image(),
+        return self.repr_card(
+            html_classes=html_classes,
+            title=title or f'{self.right.root.name} {self.right.name}{C_name}',
+            subtitle=subtitle,
+            header_href=header_href or self.right.root.name,
+            piano_html=self.to_piano_image(),
         )
 
-        return string.Template("""\
-        <div class='$classes'>
-        <a href='$href'>
-            <div class='card_header'>
-                <div class='card_title'>$title</div>
-                <div class='card_subtitle'>$subtitle</div>
-            </div>
-        </a>
-        $piano
-        </div>
-        """).substitute(mapping)
-
     def __eq__(self, other): return self.key == other.key
     def __hash__(self): return hash(self.key)
     def __repr__(self): return f'ComparedScale({self.left.root} {self.left.name} | {self.right.root} {self.right.name})'
@@ -205,13 +201,15 @@ all_scales = {
     'sudu': sudu,
 }
 
+CIRCLE_OF_FIFTHS_CLOCKWISE = 'CGDAEBfdaebF'
+
 # circle of fifths clockwise
 majors = dict(
-    diatonic=tuple(diatonic[note, 'major'] for note in 'CGDAEBfdaebF'),
-    harmonic=tuple(harmonic[note, 'h_major'] for note in 'CGDAEBfdaebF'),
-    melodic=tuple(melodic[note, 'm_major'] for note in 'CGDAEBfdaebF'),
-    pentatonic=tuple(pentatonic[note, 'p_major'] for note in 'CGDAEBfdaebF'),
-    sudu=tuple(sudu[note, 's_major'] for note in 'CGDAEBfdaebF'),
+    diatonic=tuple(diatonic[note, 'major'] for note in CIRCLE_OF_FIFTHS_CLOCKWISE),
+    harmonic=tuple(harmonic[note, 'h_major'] for note in CIRCLE_OF_FIFTHS_CLOCKWISE),
+    melodic=tuple(melodic[note, 'm_major'] for note in CIRCLE_OF_FIFTHS_CLOCKWISE),
+    pentatonic=tuple(pentatonic[note, 'p_major'] for note in CIRCLE_OF_FIFTHS_CLOCKWISE),
+    sudu=tuple(sudu[note, 's_major'] for note in CIRCLE_OF_FIFTHS_CLOCKWISE),
 )
 
 
