@@ -1,8 +1,12 @@
 from __future__ import annotations
 
+import itertools
+import pickle
 from collections.abc import Callable
 from collections.abc import Iterable
 from collections.abc import Sequence
+from concurrent.futures import ProcessPoolExecutor
+from functools import partial
 from typing import Any
 from typing import TypeVar
 
@@ -31,9 +35,22 @@ class SequenceBuilder:
         unique_key: Callable[[Op], Any] | None = None,
         loop: bool = False,
         prefix: tuple[Op, ...] = (),
+        parallel: bool = False,
     ):
+        if parallel:
+            try:
+                pickle.dumps(options)
+                pickle.dumps(options_i)
+                pickle.dumps(options_callable)
+                pickle.dumps(curr_prev_constraint)
+                pickle.dumps(candidate_constraint)
+                pickle.dumps(i_constraints)
+                pickle.dumps(unique_key)
+            except BaseException:
+                raise pickle.PickleError('parallel=True requires all arguments should be picklable')
+
         self.n = n
-        self.options: frozenset[Op] | None
+        self.options: tuple[Op, ...] | None
         self.options_i: OpsFixedPerStep[Op] | None
         self.options_callable: OpsCallableFromCurr[Op] | None
 
@@ -41,14 +58,15 @@ class SequenceBuilder:
             if not (options_i is None and options_callable is None):
                 raise OPTIONS_EXCEPTION
             options_seq = tuple(options)
-            options_fset = frozenset(options_seq)
-            if len(options_seq) != len(options_fset):
+            if len(options_seq) != len(frozenset(options_seq)):
                 raise ValueError('options should be unique')
-            self.options = options_fset
+            self.options = options_seq
             self.generate_options = self._generate_options_iterable
         elif options_i is not None:
             if not (options is None and options_callable is None):
                 raise OPTIONS_EXCEPTION
+            if len(options_i) != n:
+                raise ValueError('options_i should have options for each step up to n')
             self.options = options
             self.generate_options = self._generate_options_fixed_per_step
         elif options_callable is not None:
@@ -67,18 +85,19 @@ class SequenceBuilder:
         self.unique_key = unique_key
         self.loop = loop
         self.prefix = prefix
+        self.parallel = parallel
 
-    def _generate_options_iterable(self, seq: Sequence[Op]) -> Iterable[Op]:
+    def _generate_options_iterable(self, seq: tuple[Op, ...]) -> Iterable[Op]:
         if self.options is not None:
             return self.options
         raise TypeError
 
-    def _generate_options_fixed_per_step(self, seq: Sequence[Op]) -> Iterable[Op]:
+    def _generate_options_fixed_per_step(self, seq: tuple[Op, ...]) -> Iterable[Op]:
         if self.options_i is not None:
             return self.options_i[len(seq)]
         raise TypeError
 
-    def _generate_options_callable_from_curr(self, seq: Sequence[Op]) -> Iterable[Op]:
+    def _generate_options_callable_from_curr(self, seq: tuple[Op, ...]) -> Iterable[Op]:
         if self.options_callable is not None:
             return self.options_callable(seq[-1])
         raise TypeError
@@ -89,12 +108,13 @@ class SequenceBuilder:
     def _iter(self, prefix: tuple[Op, ...] = ()) -> Iterable[tuple[Op, ...]]:
         seq = prefix or ()
         ops = self.generate_options(seq)
+
         if self.i_constraints is not None and (i_constraint := self.i_constraints.get(len(seq))):
             ops = filter(i_constraint, ops)
 
         if self.unique_key:
             prefix_keys = frozenset(self.unique_key(op) for op in prefix)
-            ops = frozenset(op for op in ops if self.unique_key(op) not in prefix_keys)
+            ops = [op for op in ops if self.unique_key(op) not in prefix_keys]
 
         if self.curr_prev_constraint:
 
@@ -105,22 +125,36 @@ class SequenceBuilder:
                 if abs(k) > len(seq):
                     continue
                 ops = [op for op in ops if f(seq[k], op)]
+        ops = tuple(ops)
+        map_func = partial(self._generate_candidates, seq=seq)
 
-        if not prefix:
-            ops = tqdm.tqdm(ops)
+        if len(prefix) == len(self.prefix):
+            if self.parallel:
+                with ProcessPoolExecutor() as executor:
+                    for it in tqdm.tqdm(executor.map(map_func, ops), total=len(ops)):
+                        yield from it
+                return
+            else:
+                it = tqdm.tqdm(map(map_func, ops), total=len(ops))
+        else:
+            it = map(map_func, ops)
+        it = itertools.chain.from_iterable(it)
+        yield from it
 
-        for op in ops:
+    def _generate_candidates(self, op: Op, seq: tuple[Op, ...]) -> Iterable[tuple[Op, ...]]:
+        def inner():
             candidate = seq + (op,)
             if self.candidate_constraint is not None and not self.candidate_constraint(candidate):
-                continue
-            if len(candidate) == self.n:
-                if self.curr_prev_constraint and self.loop:
-                    for k, f in self.curr_prev_constraint.items():
-                        if any(not f(candidate[(i + k) % self.n], candidate[i]) for i in range(abs(k))):
-                            break
-                    else:
-                        yield tuple(candidate)
-                else:
-                    yield tuple(candidate)
-            else:
+                return
+            if len(candidate) < self.n:
                 yield from self._iter(prefix=candidate)
+                return
+            if self.curr_prev_constraint and self.loop and not all(
+                f(candidate[(i + k) % self.n], candidate[i])
+                for k, f in self.curr_prev_constraint.items()
+                    for i in range(abs(k))
+            ):
+                return
+            yield candidate
+        out = tuple(inner())
+        return out
